/*************************************************************************\
 * Copyright (c) 2002 The University of Chicago, as Operator of Argonne
 * National Laboratory.
 * Copyright (c) 2002 The Regents of the University of California, as
 * Operator of Los Alamos National Laboratory.
 * This file is distributed subject to a Software License Agreement found
 * in the file LICENSE that is included with this distribution. 
\*************************************************************************/

#include "mdb.h"
#include "SDDS.h"
#include "scan.h"
#include "column.h"

#define SET_FIXMPLNAMES 0
#define N_OPTIONS 1

char *option[N_OPTIONS] = {
  "fixMplNames"};

char *USAGE = "col2sdds <inputfile> <outputfile> [-fixMplNames]\n\
Link date: " __DATE__ " " __TIME__ ", SVN revision: " SVN_VERSION "\n";

int main(int argc, char **argv) {
  SCANNED_ARG *scanned;
  char *input = NULL, *output = NULL;
  int fixMplNames = 0;
  long i_arg, i, j, k, len;
  MC_TABLE mcTable;
  SDDS_TABLE SDDS_table;
  char buffer[100];
  char **aux_symbol = NULL;
  char **symbol = NULL;

  SDDS_RegisterProgramName(argv[0]);
  argc = scanargs(&scanned, argc, argv);
  if (argc < 3)
    bomb(NULL, USAGE);
  for (i_arg = 1; i_arg < argc; i_arg++) {
    if (scanned[i_arg].arg_type == OPTION) {
      switch (match_string(scanned[i_arg].list[0], option, N_OPTIONS, 0)) {
      case SET_FIXMPLNAMES:
        fixMplNames = 1;
        break;
      default:
        fprintf(stderr, "Invalid option seen\n%s", USAGE);
        return (1);
      }
    } else {
      if (!input)
        input = scanned[i_arg].list[0];
      else if (!output)
        output = scanned[i_arg].list[0];
      else {
        fprintf(stderr, "Too many file names\n%s", USAGE);
        return (1);
      }
    }
  }
  if (!input) {
    fprintf(stderr, "Input file not seen\n%s", USAGE);
    return (1);
  }
  if (!output) {
    fprintf(stderr, "Output file not seen\n%s", USAGE);
    return (1);
  }

  if (get_mc_table(&mcTable, input, GMCT_WARNINGS) == 0) {
    fprintf(stderr, "Unable to open %s\n", input);
    return (1);
  }

  if (fixMplNames) {
    aux_symbol = malloc(sizeof(char *) * mcTable.n_auxiliaries);
    for (i = 0; i < mcTable.n_auxiliaries; i++) {
      len = strlen(mcTable.aux_name[i]);
      aux_symbol[i] = malloc(sizeof(char) * (len + 1));
      k = 0;
      j = 0;
      while (j < len) {
        if (mcTable.aux_name[i][j] == '$') {
          j++;
        } else {
          buffer[k] = mcTable.aux_name[i][j];
          k++;
        }
        j++;
      }
      buffer[k] = 0;
      sprintf(aux_symbol[i], "%s", buffer);
    }

    symbol = malloc(sizeof(char *) * mcTable.n_cols);
    for (i = 0; i < mcTable.n_cols; i++) {
      len = strlen(mcTable.name[i]);
      symbol[i] = malloc(sizeof(char) * (len + 1));
      k = 0;
      j = 0;
      while (j < len) {
        if (mcTable.name[i][j] == '$') {
          j++;
        } else {
          buffer[k] = mcTable.name[i][j];
          k++;
        }
        j++;
      }
      buffer[k] = 0;
      sprintf(symbol[i], "%s", buffer);
    }
  }

  if (SDDS_InitializeOutput(&SDDS_table, SDDS_BINARY, 1, mcTable.title, mcTable.label, output) != 1) {
    SDDS_PrintErrors(stderr, SDDS_VERBOSE_PrintErrors);
    return (1);
  }
  for (i = 0; i < mcTable.n_auxiliaries; i++) {
    if (fixMplNames) {
      if (SDDS_DefineParameter1(&SDDS_table, aux_symbol[i], mcTable.aux_name[i], mcTable.aux_unit[i], mcTable.aux_description[i], NULL, SDDS_DOUBLE, &(mcTable.aux_value[i])) == -1) {
        SDDS_PrintErrors(stderr, SDDS_VERBOSE_PrintErrors);
        return (1);
      }
    } else {
      if (SDDS_DefineParameter1(&SDDS_table, mcTable.aux_name[i], NULL, mcTable.aux_unit[i], mcTable.aux_description[i], NULL, SDDS_DOUBLE, &(mcTable.aux_value[i])) == -1) {
        SDDS_PrintErrors(stderr, SDDS_VERBOSE_PrintErrors);
        fprintf(stderr, "Try rerunning with the -fixMplNames option\n");
        return (1);
      }
    }
  }
  for (i = 0; i < mcTable.n_cols; i++) {
    if (fixMplNames) {
      if (SDDS_DefineColumn(&SDDS_table, symbol[i], mcTable.name[i], mcTable.unit[i], mcTable.description[i], mcTable.format[i], SDDS_DOUBLE, 0) == -1) {
        SDDS_PrintErrors(stderr, SDDS_VERBOSE_PrintErrors);
        return (1);
      }
    } else {
      if (SDDS_DefineColumn(&SDDS_table, mcTable.name[i], NULL, mcTable.unit[i], mcTable.description[i], mcTable.format[i], SDDS_DOUBLE, 0) == -1) {
        SDDS_PrintErrors(stderr, SDDS_VERBOSE_PrintErrors);
        fprintf(stderr, "Try rerunning with the -fixMplNames option\n");
        return (1);
      }
    }
  }

  if (SDDS_SaveLayout(&SDDS_table) != 1) {
    SDDS_PrintErrors(stderr, SDDS_VERBOSE_PrintErrors);
    return (1);
  }
  if (SDDS_WriteLayout(&SDDS_table) != 1) {
    SDDS_PrintErrors(stderr, SDDS_VERBOSE_PrintErrors);
    return (1);
  }

  if (SDDS_StartTable(&SDDS_table, mcTable.n_rows) != 1) {
    SDDS_PrintErrors(stderr, SDDS_VERBOSE_PrintErrors);
    return (1);
  }
  for (i = 0; i < mcTable.n_cols; i++) {
    if (SDDS_SetColumnFromDoubles(&SDDS_table, SDDS_SET_BY_INDEX, mcTable.value[i], mcTable.n_rows, i) != 1) {
      SDDS_PrintErrors(stderr, SDDS_VERBOSE_PrintErrors);
      return (1);
    }
  }
  if (SDDS_WriteTable(&SDDS_table) != 1) {
    SDDS_PrintErrors(stderr, SDDS_VERBOSE_PrintErrors);
    return (1);
  }
  if (SDDS_Terminate(&SDDS_table) != 1) {
    SDDS_PrintErrors(stderr, SDDS_VERBOSE_PrintErrors);
    return (1);
  }

  return (0);
}
